/*
 * Base types used by various other types.
 *
 * Copyright (C) 2019, 2021, 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/compiler.h>
#include <l4/sys/types.h>



EXTERN_C_BEGIN

/* Types for file access (access flags, offsets) and IPC operations. */

typedef l4_uint64_t         flags_t;        /* see systypes/fcntl.h */
typedef l4_uint64_t         offset_t;
typedef l4_addr_t           address_t;      /* unsigned long */

/* Dataspace mapping flags.
   See: pkg/l4re-core/l4re_c/include/dataspace.h */

typedef l4_uint64_t         map_address_t;
typedef unsigned long       map_flags_t;

/* Conversions. */

map_flags_t map_flags_for_fault(l4_umword_t flags);

/* Types and values for notification. */

typedef l4_uint64_t         notify_flags_t;

typedef struct
{
  unsigned long sig;                        /* signal number */
  long val;                                 /* signal-specific value */

} notify_values_t;

#define NOTIFY_VALUES_NULL ((notify_values_t) {0, 0})

enum notify_flags
{
  NOTIFY_CONTENT_AVAILABLE  = 0x001,        /* reading files and pipes */
  NOTIFY_SPACE_AVAILABLE    = 0x002,        /* writing pipes */
  NOTIFY_PEER_CLOSED        = 0x004,        /* closing files and pipes */
  NOTIFY_FILE_OPENED        = 0x008,        /* opening files in directories */
  NOTIFY_TASK_SIGNAL        = 0x100,        /* signal from task */
  NOTIFY_TASK_ERROR         = 0x200,        /* error when creating task */

  /* Combinations of flags. */

  NOTIFY_TASK_ALL           = 0x300,
};

/* Notifiable object types. */

typedef struct
{
  l4_cap_idx_t ref;

} notifiable_base_t;

typedef struct
{
  notifiable_base_t *base;                  /* access to the specific object */
  notify_flags_t notifications;             /* essential notifications */
  notify_flags_t pending_notifications;
  notify_values_t values;                   /* signal-specific values */
  notify_values_t pending_values;
  void *handler;                            /* associated notification handler */

} notifiable_t;

/* Filesystem object properties. */

typedef unsigned long       object_flags_t;

enum object_flags
{
  OBJECT_SUPPORTS_MMAP      = 1,
  OBJECT_HAS_SIZE           = 2
};

/* Memory mapping protection flags compatible with sys/mman.h (and incompatible
   with comparable L4Re flags). */

typedef unsigned long       prot_t;

enum prot_flags
{
  PROT_NONE                 = 0,
  PROT_READ                 = 1,
  PROT_WRITE                = 2,
  PROT_EXEC                 = 4
};

/* Equivalent types are defined in sys/types.h typically. In newlib, they are
   defined in sys/_types.h if not defined elsewhere (such as in
   machine/_types.h). In uclibc, the following file is informative:

   pkg/l4re-core/uclibc/lib/contrib/uclibc/libc/sysdeps/linux/common/bits/types.h

   These types are intended for IPC, permitting different parties to employ
   different library implementations where these types may be different.
*/

typedef l4_uint64_t         sys_dev_t;      /* device number */
typedef unsigned long int   sys_ino_t;      /* inode number */
typedef unsigned int        sys_mode_t;     /* file permissions */
typedef unsigned long int   sys_nlink_t;    /* link count */
typedef unsigned int        sys_uid_t;      /* user identifier */
typedef unsigned int        sys_gid_t;      /* group identifier */
typedef long                sys_off_t;      /* file offset/position */
typedef long                sys_blksize_t;  /* input/output block size */
typedef long                sys_blkcnt_t;   /* number of 512-byte blocks */

/* Conversions. */

#define systypes_from_sys_mode(mode) \
((mode_t) mode)

#define systypes_to_sys_mode(mode) \
((sys_mode_t) mode)

EXTERN_C_END

// vim: tabstop=2 expandtab shiftwidth=2
